package com.tzld.piaoquan.ad.engine.service.predict.model.threshold;

import com.tzld.piaoquan.ad.engine.commons.redis.AlgorithmRedisHelper;
import com.tzld.piaoquan.ad.engine.commons.score.AdConfig;
import com.tzld.piaoquan.ad.engine.commons.score.ScoreParam;
import com.tzld.piaoquan.ad.engine.commons.score.ScorerUtils;
import com.tzld.piaoquan.ad.engine.commons.util.CommonCollectionUtils;
import com.tzld.piaoquan.ad.engine.service.predict.param.RuleParamHelper;
import com.tzld.piaoquan.ad.engine.service.predict.param.ThresholdPredictModelParam;
import com.tzld.piaoquan.ad.engine.service.remote.FeatureRemoteService;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.AdRankItem;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.AdRequestContext;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.UserAdFeature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class ScoreV2ThresholdPredictModel extends ThresholdPredictModel {

    @Autowired
    private FeatureRemoteService featureRemoteService;

    @Value("${ad.predict.threshold:1}")
    private double threshold;

    @Override
    String initName() {
        return "modelV2";
    }

    @Override
    public Map<String, Object> predict(ThresholdPredictModelParam modelParam) {

        UserAdFeature userAdFeature = featureRemoteService.getUserAdFeature(modelParam.getMid());
        if (userAdFeature == null) {
            userAdFeature = new UserAdFeature();
        }
        List<AdRankItem> rankItems = featureRemoteService.
                getAllAdFeatureList(
                        CommonCollectionUtils.toList(AdConfig.getAdIds(), id -> id.toString())
                );

        // scoreParam
        AdRequestContext context = new AdRequestContext();
        context.setApptype(modelParam.getAppType().toString());
        context.setMachineinfoBrand(modelParam.getMachineInfo().getBrand());
        context.setMachineinfoModel(modelParam.getMachineInfo().getModel());
        context.setMachineinfoSdkversion(modelParam.getMachineInfo().getSdkVersion());
        context.setMachineinfoWchatversion(modelParam.getMachineInfo().getWeChatVersion());
        LocalDateTime date = LocalDateTime.now();
        context.setHour(date.getHour() + "");
        context.setDay(date.format(DateTimeFormatter.ofPattern("yyyyMMdd")));
        context.setWeek(date.getDayOfWeek().getValue() + "");

        ScoreParam scoreParam = new ScoreParam();
        scoreParam.setRequestContext(context);
        scoreParam.getRequestContext().setRegion(modelParam.getRegion().replace("省", ""));
        scoreParam.getRequestContext().setCity(modelParam.getCity().replace("市", ""));


        List<AdRankItem> scoreResult = ScorerUtils
                .getScorerPipeline("feeds_score_config_break")
                .scoring(scoreParam, userAdFeature, rankItems);

        // 找出ctr*cvr最大的
        double max = -1;
        AdRankItem maxItem = null;
        for (int i = 0; i < scoreResult.size(); i++) {
            AdRankItem item = scoreResult.get(i);
            double ctrCvr = item.getCtr() * item.getCvr();
            if (ctrCvr > max) {
                max = ctrCvr;
                maxItem = item;
            }
        }

        int adPredict;
        if (maxItem.getScore() < threshold) {
            // If final score is below threshold, show the ad
            adPredict = 2;
        } else {
            // Otherwise, do not show the ad
            adPredict = 1;
        }
        Map<String, Object> result = new HashMap<>();
        result.put("ad_predict", adPredict);
        return result;
    }
}
